//---------------------------------------------------------------------------
// MACD_PowerM indicator - coded by wokraczka@hotmail.com
// Modified to MACD_BD_Color by Christoff
//---------------------------------------------------------------------------
library MACD_BD_Color;

uses
  SysUtils, classes, graphics, windows, TechnicalFunctions, IndicatorInterfaceUnit;

var
  // External variables
  FastEMAPeriod: integer;
  SlowEMAPeriod: integer;
  SMAPeriod: integer;
  EMAPeriod: integer;
  ApplyToPrice: integer;
  histG: TIndexBuffer;
  histR: TIndexBuffer;
  histB: TIndexBuffer;
  histY: TIndexBuffer;

  // Buffers
  FastEMA, SlowEMA, _MACD, SMA, EMA: TIndexBuffer;

//---------------------------------------------------------------------------
// Initialize indicator
//---------------------------------------------------------------------------
procedure Init; stdcall;
begin
  // define properties
  IndicatorShortName('MACD_BD_Color');
  SetOutputWindow(ow_SeparateWindow);
  AddLevel(0, psDot, 1, cl_GridColor);

  // register options
  AddSeparator('Coded by wokraczka@hotmail.com');
  AddSeparator('MACD_BD_Color');

  RegOption('Fast EMA period', ot_Integer, FastEMAPeriod);
  SetOptionRange('Fast EMA period', 1, MaxInt);
  FastEMAPeriod := 35;

  RegOption('Slow EMA period', ot_Integer, SlowEMAPeriod);
  SetOptionRange('Slow EMA period', 1, MaxInt);
  SlowEMAPeriod := 70;

  RegOption('SMA period', ot_Integer, SMAPeriod);
  SetOptionRange('SMA period', 1, MaxInt);
  SMAPeriod := 1;

  RegApplyToPriceOption(ApplytoPrice,'Apply To Price');
  ApplyToPrice:= 0;

  RegOption('EMA of Signal period', ot_Integer, EMAPeriod);
  SetOptionRange('EMA of Signal period', 1, MaxInt);
  EMAPeriod := 12;

  // create buffers
  FastEMA := CreateIndexBuffer;
  SlowEMA := CreateIndexBuffer;
  _MACD := CreateIndexBuffer;
  SMA := CreateIndexBuffer;
  histG := CreateIndexBuffer;
  histR := CreateIndexBuffer;
  histB := CreateIndexBuffer;
  histY := CreateIndexBuffer;
  EMA := CreateIndexBuffer;

  IndicatorBuffers(7);
  SetIndexBuffer(0, _MACD);
  SetIndexBuffer(1, SMA);
  SetIndexStyle(0, ds_Histogram, psSolid, 1, clSilver);
  SetIndexLabel(0, 'MACD');
  SetIndexStyle(1, ds_Line, psSolid, 2, clRed);
  SetIndexLabel(1, 'Signal Line');
  SetIndexBuffer(2, histG);
  SetIndexStyle(2, ds_Histogram, psSolid, 1, clYellow);
  SetIndexLabel(2, 'Positive Up');
  SetIndexBuffer(3, histR);
  SetIndexStyle(3, ds_Histogram, psSolid, 1, clGreen);
  SetIndexLabel(3, 'Positive Down');
  SetIndexBuffer(4, histB);
  SetIndexStyle(4, ds_Histogram, psSolid, 1, clBlue);
  SetIndexLabel(4, 'Negative Up');
  SetIndexBuffer(5, histY);
  SetIndexStyle(5, ds_Histogram, psSolid, 1, clRed);
  SetIndexLabel(5, 'Negative Down');
  SetIndexBuffer(6, EMA);
  SetIndexStyle(6, ds_Line, psSolid, 2, clBlue);
  SetIndexLabel(6, 'EMA of Signal Line');
end;

//---------------------------------------------------------------------------
// Deinitialize indicator
//---------------------------------------------------------------------------
procedure Done; stdcall;
begin

end;

  // function used to calculate EMA of Signal Line
  function GetEMAOfBuffer(index: integer; buffer: TIndexBuffer; period: integer; prev: double): double;
  var
    k: double;
    i: integer;
  begin
    // EMA
    k := 2/(period + 1);
    i := index;
    if (i > Bars - 1) or (i < 0) then
      result := 0
    else
      begin
        if prev = 0 then
        result := buffer[i]
      else
        result := prev + k*(buffer[i] - prev);
      end;
    end;

//---------------------------------------------------------------------------
// Calculate requested bar
//---------------------------------------------------------------------------
procedure Calculate(index: integer); stdcall;
var
  i : integer;
  k, sum : double;

  function GetPrice(index: integer): double;
  begin
    case ApplyToPrice of
      0:   result := Close(index);
      1:   result := Open(index);
      2:   result := High(index);
      3:   result := Low(index);
      4:   result := (High(index) + Low(index))/2;
      5:   result := (High(index) + Low(index) + Close(index))/3;
      6:   result := (High(index) + Low(index) + Close(index)*2)/4;
      else result := 0;
    end;
  end;

begin
  // calculate fast EMA
  k := 2/(FastEMAPeriod + 1);
  if index = Bars - 1 then
    FastEMA[index] := GetPrice(index)
  else
    FastEMA[index] := FastEMA[index + 1] + k*(GetPrice(index) - FastEMA[index + 1]);

  // calculate slow EMA
  k := 2/(SlowEMAPeriod + 1);
  if index = Bars - 1 then
    SlowEMA[index] := GetPrice(index)
  else
    SlowEMA[index] := SlowEMA[index + 1] + k*(GetPrice(index) - SlowEMA[index + 1]);

  // calculate MACD
  _MACD[index] := FastEMA[index] - SlowEMA[index];
//---------------------------------------------------------------------------
// MACD_BD_Color calculations
//---------------------------------------------------------------------------
  if (_MACD[index]>_MACD[index+1]) and (_MACD[index]>0) Then
  begin
  histG[index] := _MACD[index];
  histR[index] := 0;
  histB[index] := 0;
  histY[index] := 0;
  end
  else if (_MACD[index]<_MACD[index+1]) and (_MACD[index]>0) Then
  begin
  histG[index] := 0;
  histR[index] := _MACD[index];
  histB[index] := 0;
  histY[index] := 0;
  end
  else if (_MACD[index]<_MACD[index+1]) and (_MACD[index]<0) Then
  begin
  histG[index] := 0;
  histR[index] := 0;
  histB[index] := _MACD[index];
  histY[index] := 0;
  end
  else if (_MACD[index]>_MACD[index+1]) and (_MACD[index]<0) Then
  begin
  histG[index] := 0;
  histR[index] := 0;
  histB[index] := 0;
  histY[index] := _MACD[index];
  end;
  // calculate SMA
  sum := 0;
  for i:=index to index + SMAPeriod - 1 do
    sum := sum + _MACD[i];
  SMA[index] := sum/SMAPeriod;

  EMA[index] := GetEMAOfBuffer(index, SMA, EMAPeriod, EMA[index + 1]);
end;

exports

Init, Done, Calculate;

end.

